import unittest
import os

from gym_mapf.envs.grid import MapfGrid
from gym_mapf.envs import UP, DOWN, LEFT, RIGHT, STAY
from gym_mapf.envs.mapf_env import execute_action
from gym_mapf.envs.utils import parse_map_file
from gym_mapf.tests import MAPS_DIR


# TODO: Make this test not depend on the file system.
class ExecutorTest(unittest.TestCase):
    def test_moving_on_empty_grid(self):
        map_file_path = os.path.abspath(os.path.join(__file__, MAPS_DIR, 'empty-8-8/empty-8-8.map'))
        grid = MapfGrid(parse_map_file(map_file_path))

        s = ((0, 0), (7, 7))

        new_state = execute_action(grid, s, (RIGHT, UP))
        self.assertEqual(new_state, ((0, 1), (6, 7)))

        new_state = execute_action(grid, s, (DOWN, LEFT))
        self.assertEqual(new_state, ((1, 0), (7, 6)))

    def test_against_the_wall(self):
        map_file_path = os.path.abspath(os.path.join(__file__, MAPS_DIR, 'empty-8-8/empty-8-8.map'))
        grid = MapfGrid(parse_map_file(map_file_path))

        s = ((0, 0), (7, 7))

        new_state = execute_action(grid, s, (LEFT, RIGHT))
        self.assertEqual(new_state, ((0, 0), (7, 7)))

    def test_against_obstacle_stays_in_place(self):
        grid = MapfGrid([
            '..@..',
            '..@..',
            '.....',
            '..@..',
            '..@..'])

        s = ((0, 1),)  # start near an obstacle.

        new_state = execute_action(grid, s, (RIGHT,))
        self.assertEqual(new_state, ((0, 1),))  # The agent hits an obstacle and should stay in place.

    def test_stay_action(self):
        map_file_path = os.path.abspath(os.path.join(__file__, MAPS_DIR, 'empty-8-8/empty-8-8.map'))
        grid = MapfGrid(parse_map_file(map_file_path))

        s = ((0, 0), (7, 7))

        new_state = execute_action(grid, s, (STAY, STAY))
        self.assertEqual(new_state, ((0, 0), (7, 7)))


if __name__ == '__main__':
    unittest.main(verbosity=2)



import os

MAPS_PATH = os.path.abspath(os.path.join(__file__, '../../maps/'))


def map_name_to_files(map_name, scen_id):
    map_file_path = os.path.join(MAPS_PATH, map_name, '{}.map'.format(map_name))
    scen_file_path = os.path.join(MAPS_PATH, map_name, '{}-even-{}.scen'.format(map_name, scen_id))

    return map_file_path, scen_file_path


UP = 'UP'
RIGHT = 'RIGHT'
DOWN = 'DOWN'
LEFT = 'LEFT'
STAY = 'STAY'
POSSIBILITIES = {
    UP: (RIGHT, LEFT),
    DOWN: (LEFT, RIGHT),
    LEFT: (UP, DOWN),
    RIGHT: (DOWN, UP),
    STAY: (STAY, STAY)
}
ACTIONS = [STAY, UP, RIGHT, DOWN, LEFT]
ACTIONS_TO_INT = {
    a: ACTIONS.index(a) for a in ACTIONS
}


def integer_to_vector(x, options_per_element, n_elements, index_to_element):
    """Return a vector representing an action/state from a given integer.

    Args:
        x (int): the integer to convert.
        n_options_per_element(int): number of options for each element in the vector.
        n_elements (int): the number of elements in the vector to return.
        index_to_element(int=>any): function which converts an integer represents a single option in one of the
            vector elements and return anything that vector contains. For example, a function which returns 'UP' for 0,
            1 for 'RIGHT',etc. Or a function which returns (2,2) given 10 for a 4x4 grid ((2,2) is the 10-th cell of that grid).
    """
    return integer_to_vector_multiple_numbers(x, options_per_element, n_elements, index_to_element)


def vector_to_integer(v, options_per_element, element_to_index):
    return vector_to_integer_multiple_numbers(v, options_per_element, element_to_index)


def integer_to_vector_multiple_numbers(x, n_options_per_element, n_elements, index_to_element):
    """Return a vector representing an action/state from a given integer.

    Args:
        x (int): the integer to convert.
        n_options_per_element(list): number of options for each element in the vector.
        n_elements (int): the number of elements in the vector to return.
        index_to_element(int=>any): function which converts an integer represents a single option in one of the
            vector elements and return anything that vector contains. For example, a function which returns 'UP' for 0,
            1 for 'RIGHT',etc. Or a function which returns (2,2) given 10 for a 4x4 grid ((2,2) is the 10-th cell of that grid).
    """
    ret = tuple()
    for i in range(0, n_elements):
        option_index = x % n_options_per_element[i]
        ret = ret + (index_to_element(option_index),)
        x //= n_options_per_element[i]

    return ret


def vector_to_integer_multiple_numbers(v, n_options_per_element, element_to_index):
    sum = 0
    mul = 1
    for i in range(len(v)):
        if i != 0:
            mul *= n_options_per_element[i - 1]

        sum += element_to_index(v[i]) * mul

    return sum



import setuptools
import os

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    long_description = readme.read()

setuptools.setup(
    name="gym-mapf",
    version="0.4.2",
    author="LevyvoNet",
    author_email="eladlevy2@gmail.com",
    description="Multi-Agent Path Finding gym environment",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/LevyvoNet/gym-mapf",
    packages=setuptools.find_packages(),
    include_package_data=True,
    test_suite="gym_mapf.tests",
    classifiers=[
        "Programming Language :: Python :: 3.6",
    ],
)



import time
import functools


def measure_time(func, *args, **kwargs):
    @functools.wraps(func)
    def new_func(*args, **kwargs):
        before = time.time()
        func(*args, **kwargs)
        after = time.time()

        print(f'took {after - before} seconds')

    return new_func
